import { apiWaypointEntity } from "../lib/apiwaypoint/waypointEntity.js"
import { world, system } from "@minecraft/server"
import { apiFormat } from "../lib/apiFormat.js"
import { apiInfo } from "../lib/apiInfo.js"

system.runInterval(() => {
    world.getAllPlayers().forEach(player => {
        const config = apiInfo.getWaypointConfig(player)
        const playerPos = player.location
        for(const entity of player.dimension.getEntities({type: "wp:waypoint", tags: [`waypointOwner: ${player.id}`]})){
            const waypointPos = entity.getDynamicProperty("waypointPos")
            if(!waypointPos){ apiWaypointEntity.removeWaypoint(player, entity); continue }
            if(player.dimension.id != entity.getTags().find(tag => tag.startsWith("waypointWorld: "))?.replace("waypointWorld: ", "")) return apiWaypointEntity.removeWaypoint(player, entity)
            try{
                if(resetEntity(player, entity)) system.runTimeout(() => { apiWaypointEntity.recoverWaypoints(player) }, 20 * 2.5)
                entity.tryTeleport({x: playerPos.x, y: playerPos.y - 10, z: playerPos.z})
                const distanceX = playerPos.x - waypointPos.x, distanceZ = playerPos.z - waypointPos.z
                const distance = Math.sqrt(distanceX ** 2 + distanceZ ** 2)
                if(distance > 7){
                    const radius = distance > 64 ? 64 : distance
                    const degress = Math.floor(Math.atan2(distanceZ, distanceX) * (180 / Math.PI))
                    const radians = degress * (Math.PI / 180)

                    const offsetX = Math.sqrt((radius * Math.cos(radians)) ** 2)
                    const offsetZ = Math.sqrt((radius * Math.sin(radians)) ** 2)
                    const offset = offsetZ < (radius * Math.sin(45 * (Math.PI / 180))) ? offsetX : offsetZ
                    const scale = distance == radius ? apiFormat.clamp((radius / 4) / 6.4, 0, 2) : 2

                    if(config.showDistance) apiFormat.setDisplayDistance(entity, Math.floor(distance))
                    !config.showName ? entity.setProperty("wp:offsetdistancey", +13.5) : entity.setProperty("wp:offsetdistancey", 0)
                    entity.setRotation({x: 0, y: degress -90})
                    entity.setProperty("wp:scale", scale)
                    entity.setProperty("wp:offset", offset + scale * 9)
                    entity.playAnimation("animation.waypoint.offset", {players: [`${player.nameTag}`]})
                }
            } catch {}
        }
    })
}, 1)

function resetEntity(player, entity){
    const playerPos = player.location
    const entityPos = entity.location
    const distance =  Math.sqrt((playerPos.x - entityPos.x) ** 2 + (playerPos.z - entityPos.z) ** 2)
    if(distance > 5) entity.teleport(playerPos)
    if(distance > 64) return true
}