import { world } from "@minecraft/server"
import { maxXpCost } from "../variables"

const offsetDistance = [0, 12.5, 9.75, 6.75, 4, 1.15, -1.5]
const offsetName = [19.25, 16.5, 13.75, 11, 8.25, 5.5, 2.75, 0]
const defaultLetters = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "(", ")", ":"]
const replaceLetters = ["ã", "á", ""]

export const apiFormat = new class apiFormat {
    formatMiddlePos(pos, textField = false){
        if(textField) return `${Math.floor(pos.x)}, ${Math.floor(pos.z)}`
        return {x: Math.floor(pos.x) +0.5, y: Math.floor(pos.y), z: Math.floor(pos.z) +0.5}
    }

    formatPos(pos){ return {x: Math.floor(pos.x), y: Math.floor(pos.y), z: Math.floor(pos.z)} }

    clamp(num, min, max){ return Math.min(Math.max(num, min), max) }

    setDisplayDistance(waypoint, distance){
        const letters = `${Math.floor(distance > 99999 ? 99999 : distance)}`.split("")
        const length = letters.length
        waypoint.setProperty(`wp:offsetdistance`, offsetDistance[length + (distance > 99999 ? 1 : 0)])
        waypoint.setProperty(`wp:distance2`, -1)
        waypoint.setProperty(`wp:distance3`, -1)
        waypoint.setProperty(`wp:distance4`, -1)
        waypoint.setProperty(`wp:distance5`, -1)
        waypoint.setProperty(`wp:distance6`, distance > 99999 ? 0 : -1)
        for(let i = 0; i < length; i++){
            waypoint.setProperty(`wp:distance${i +1}`, !letters[length - i -1] ? -1 : parseInt(letters[length - i -1]))
        }
    }

    setDisplayName(waypoint, name){
        const letters = name.slice(0, 8).normalize("NFD").replace(/[\u0300-\u036f]/g, "")
        const length = letters.length
        waypoint.setProperty(`wp:offsetname`, offsetName[length -1])
        for(let i = 0; i < length; i++){
            waypoint.setProperty(`wp:name${length - i}`, defaultLetters.findIndex(l => letters[i].toLowerCase() == l))
        }
    }

    calculateCost(player, waypoint){
        if(maxXpCost() == 0) return 0
        const playerPos = player.location
        const waypointPos = waypoint.pos
        const distance = Math.floor(Math.sqrt((waypointPos.x - playerPos.x) ** 2  + (waypointPos.z - playerPos.z) ** 2)) / 1500
        return distance < maxXpCost() ? Math.floor(distance) : maxXpCost()
    }
}