import { world, Entity, Player } from "@minecraft/server"
import { apiInfo } from "../apiInfo.js"
import { apiFormat } from "../apiFormat.js"

export const apiWaypointEntity = new class apiWaypointEntity {
    /** @param {Player} player @remakes Recover all waypoint entity.*/
    recoverWaypoints(player){
        this.removeAllWaypoints(player)
        const waypoints = apiInfo.getAllWaypoints(player)
        waypoints.forEach(waypoint => { if(waypoint.visible) this.spawnWaypoint(player, waypoint) })
    }

    /** @param {Player} player @remakes Remove all waypoint entity with the player ID.*/
    removeAllWaypoints(player){
        player.dimension.getEntities({tags: [`waypointOwner: ${player.id}`]}).forEach(way => way.remove())
    }

    /** @param {Player} player @param {string | Entity} waypoint @remakes Remove one waypoint entity by waypoint name or waypoint object.*/
    removeWaypoint(player, waypoint){
        if(typeof waypoint == "string") return player.dimension.getEntities({type: "wp:waypoint", tags: [`waypointName: ${waypoint}`, `waypointOwner: ${player.id}`]})[0]?.remove()
        waypoint.remove()
    }

    /** @param {Player} player @param {string | Entity} waypoint @remakes Summon one waypoint entity by waypoint info.*/
    spawnWaypoint(player, waypoint){
        const config = apiInfo.getWaypointConfig(player)
        const rgb = waypoint.rgb
        const entity = player.dimension.spawnEntity("wp:waypoint", player.location)
        entity.addTag(`waypointOwner: ${player.id}`)
        entity.addTag(`waypointName: ${waypoint.id}`)
        entity.addTag(`waypointWorld: ${waypoint.world}`)
        entity.setProperty("wp:icon", waypoint.icon)
        entity.setProperty("wp:r", rgb.r)
        entity.setProperty("wp:g", rgb.g)
        entity.setProperty("wp:b", rgb.b)
        if(config.showName) apiFormat.setDisplayName(entity, waypoint.id)
        entity.setDynamicProperty("waypointPos", waypoint.pos)
    }
}