import { apiWaypointEntity } from "../lib/apiwaypoint/waypointEntity.js"
import { ModalFormData } from "@minecraft/server-ui"
import { apiInfo } from "../lib/apiInfo.js"
import { world } from "@minecraft/server"

export const dateType = [
    "ss:mm:hh - MM:DD:YY", "ss:mm:hh - DD:MM:YY", "ss:mm:hh - YY:MM:DD",
    "ss:mm:hh - MM:DD:YYYY", "ss:mm:hh - DD:MM:YYYY", "ss:mm:hh - YYYY:MM:DD"
]

export const waypointOptions = new class waypointOptions {
    // General Settings
    0(player){
        const config = apiInfo.getWaypointConfig(player)
        new ModalFormData()
        .title("waypoint.ui.options.general.title")
        .toggle("waypoint.ui.options.general.toggle.showName", config.showName)
        .toggle("waypoint.ui.options.general.toggle.showDistance", config.showDistance)
        .toggle("waypoint.ui.options.general.toggle.showPos", config.showPos ?? true)
        .toggle("waypoint.ui.options.general.toggle.createDeathPoint", config.createDeathpoint)
        .dropdown("waypoint.ui.options.general.dropdown.deathPointDate", dateType, config.deathPointDate)
        .submitButton("waypoint.ui.create.buttonSave")
        .show(player).then(r => {
            if(r.canceled){
                player.playSound("random.break")
                return player.onScreenDisplay.setActionBar({translate: "waypoint.warning.cancelOptions"})
            }
            config.showName = r.formValues[0]
            config.showDistance = r.formValues[1]
            config.showPos = r.formValues[2]
            config.createDeathpoint = r.formValues[3]
            config.deathPointDate = r.formValues[4]
            player.setDynamicProperty("waypointConfig", JSON.stringify(config))
            player.onScreenDisplay.setActionBar({translate: "waypoint.warning.saveOptions"})
            player.playSound("random.levelup")
            apiWaypointEntity.recoverWaypoints(player)
            if(!config.showDistance){
                player.dimension.getEntities({tags: [`waypointOwner: ${player.id}`]})?.forEach(way => {
                    for(let i = 0; i < 5; i++) way.setProperty(`wp:distance${i +1}`, -1)
                })
            }
        })
    }

    // Recover Player's Waypoints
    1(player){
        apiWaypointEntity.recoverWaypoints(player)
    }
}